@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2011 Roberto Bordolanghi
+# Copyright (C) 2010-2012 Roberto Longobardi
 # 
 # This file is part of the Test Manager plugin for Trac.
 # 
@@ -138,16 +138,18 @@ class TestManagerSystem(Component):
             for outcome in self.outcomes_by_color[color]:
                 self.outcomes_by_name[outcome] = [color, self.outcomes_by_color[color][outcome]]
 
-    def get_next_id(self, type):
-        propname = self.NEXT_PROPERTY_NAME[type]
-    
-        # Get current latest ID for the desired object type
-        latest_id = self.get_config_property(propname)
-        if not latest_id:
-            latest_id = '0'
+    def get_next_id(self, type_):
+        latest_id = -1
+        if type_ in self.NEXT_PROPERTY_NAME:
+            propname = self.NEXT_PROPERTY_NAME[type_]
+        
+            # Get current latest ID for the desired object type
+            latest_id = self.get_config_property(propname)
+            if not latest_id:
+                latest_id = '0'
 
-        # Increment next ID
-        self.set_config_property(propname, str(int(latest_id)+1))
+            # Increment next ID
+            self.set_config_property(propname, str(int(latest_id)+1))
 
         return latest_id
     
@@ -157,7 +159,7 @@ class TestManagerSystem(Component):
 
     def get_config_property(self, propname):
         try:
-            db = get_db(self.env)
+            db = self.env.get_read_db()
             cursor = db.cursor()
             sql = "SELECT value FROM testconfig WHERE propname=%s"
             
@@ -176,8 +178,8 @@ class TestManagerSystem(Component):
             return None
     
     def set_config_property(self, propname, value):
-        db, handle_ta = get_db_for_write(self.env)
-        try:
+        @with_transaction(self.env)
+        def do_set_config_property(db):
             cursor = db.cursor()
             sql = "SELECT COUNT(*) FROM testconfig WHERE propname = %s"
             cursor.execute(sql, (propname,))
@@ -193,17 +195,8 @@ class TestManagerSystem(Component):
                                INSERT INTO testconfig (propname,value)
                                    VALUES (%s,%s)
                                """, (propname, str(value)))
-            if handle_ta:
-                db.commit()
  
-            return True
- 
-        except:
-            self.env.log.error("Error setting configuration property '%s' to '%s'" % (propname, str(value)))
-            self.env.log.error(formatExceptionInfo())
-            db.rollback()
-
-        return False
+        return True
     
     def get_default_tc_status(self):
         """Returns the default test case in plan status"""
@@ -247,7 +240,7 @@ class TestManagerSystem(Component):
         result += '<tr><th>'+_("Timestamp")+'</th><th>'+_("Author")+'</th><th>'+_("Status")+'</th></tr>'
         result += '</thead><tbody>'
         
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
 
         sql = "SELECT time, author, status FROM testcasehistory WHERE id='"+str(id)+"' AND planid='"+str(planid)+"' ORDER BY time DESC"
@@ -269,7 +262,7 @@ class TestManagerSystem(Component):
     def list_all_testplans(self):
         """Returns a list of all test plans."""
 
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
 
         sql = "SELECT id, catid, page_name, name, author, time FROM testplan ORDER BY catid, id"
@@ -295,11 +288,11 @@ class TestManagerSystem(Component):
             match = True
         
         if req.path_info.startswith('/testcreate') and (((type == 'catalog' or type == 'testcase') and ('TEST_MODIFY' in req.perm)) or 
-             (type == 'testplan' and ('TEST_PLAN_ADMIN' in req.perm))):
+             ((type == 'testplan' or type == 'testcaseinplan') and ('TEST_PLAN_ADMIN' in req.perm))):
             match = True
         elif (req.path_info.startswith('/teststatusupdate') and 'TEST_EXECUTE' in req.perm):
             match = True
-        elif (req.path_info.startswith('/testdelete') and type == 'testplan' and 'TEST_PLAN_ADMIN' in req.perm):
+        elif (req.path_info.startswith('/testdelete') and (type == 'testplan' or type == 'testcaseinplan') and 'TEST_PLAN_ADMIN' in req.perm):
             match = True
         elif (req.path_info.startswith('/testimport') and ('TEST_MODIFY' in req.perm)):
             match = True
@@ -377,7 +370,7 @@ class TestManagerSystem(Component):
                 except:
                     self.env.log.error("Error adding test catalog!")
                     self.env.log.error(formatExceptionInfo())
-                    add_notice(req, _("Error adding test catalog"))
+                    add_warning(req, _("An error occurred while adding the test catalog."))
                     req.redirect(req.href.wiki(path))
 
                 # Redirect to see the new wiki page.
@@ -385,33 +378,99 @@ class TestManagerSystem(Component):
                 
             elif object_type == 'testplan':
                 req.perm.require('TEST_PLAN_ADMIN')
+                is_update = req.args.get('update', 'false')
 
-                contains_all_str = req.args.get('containsAll', 'true')
-                snapshot_str = req.args.get('snapshot', 'false')
-                selected_tcs_str = req.args.get('selectedTCs', '')
-
-                contains_all = (0, 1)[contains_all_str == 'true']
-                snapshot = (0, 1)[snapshot_str == 'true']
-                selected_tcs = []
-                if contains_all_str == 'false' and not selected_tcs_str == '':
-                    selected_tcs = selected_tcs_str.split(',')
-
-                catid = path.rpartition('_TT')[2]
-
-                try:
-                    # Add the new test plan in the database
-                    new_tc = TestPlan(self.env, id, catid, pagename, title, author, contains_all, snapshot, selected_tcs)
-                    new_tc.insert()
-
-                except:
-                    self.env.log.error("Error adding test plan!")
-                    self.env.log.error(formatExceptionInfo())
-                    # Back to the catalog
-                    req.redirect(req.href.wiki(path))
-
-                # Display the new test plan
-                req.redirect(req.href.wiki(path, planid=str(id)))
+                if is_update == 'true':
+                    planId = req.args.get('planid')
                     
+                    try:
+                        # Update the version of the test case in plan to 
+                        # the latest wiki page version
+                        pagename = tcId
+                        id = tcId.rpartition('_TC')[2]
+                        tcip = TestCaseInPlan(self.env, id, planId, pagename)
+                        tcip.update_version()
+                        tcip.save_changes(author, "Version updated")
+                        add_notice(req, _("The test case version was updated successfully."))
+                        
+                    except:
+                        self.env.log.error("Error updating the test case version!")
+                        self.env.log.error(formatExceptionInfo())
+                        add_warning(req, _("An error occurred while updating the test case version."))
+                
+                    # Display the updated test case in plan
+                    req.redirect(req.href.wiki(tcId, planid=planId))
+                    
+                else:
+                    contains_all_str = req.args.get('containsAll', 'true')
+                    snapshot_str = req.args.get('snapshot', 'false')
+                    selected_tcs_str = req.args.get('selectedTCs', '')
+
+                    contains_all = (0, 1)[contains_all_str == 'true']
+                    snapshot = (0, 1)[snapshot_str == 'true']
+                    selected_tcs = []
+                    if contains_all_str == 'false' and not selected_tcs_str == '':
+                        selected_tcs = selected_tcs_str.split(',')
+
+                    catid = path.rpartition('_TT')[2]
+
+                    try:
+                        # Add the new test plan in the database
+                        new_tc = TestPlan(self.env, id, catid, pagename, title, author, contains_all, snapshot, selected_tcs)
+                        new_tc.insert()
+
+                    except:
+                        self.env.log.error("Error adding test plan!")
+                        self.env.log.error(formatExceptionInfo())
+                        # Back to the catalog
+                        add_warning(req, _("An error occurred while generating the test plan."))
+                        req.redirect(req.href.wiki(path))
+
+                    # Display the new test plan
+                    req.redirect(req.href.wiki(path, planid=str(id)))
+                    
+            elif object_type == 'testcaseinplan':
+                req.perm.require('TEST_PLAN_ADMIN')
+                
+                tcatId = req.args.get('tcatId')
+                planid = req.args.get('planid')
+                self.env.log.debug("About to add test case %s to test plan %s" % (tcId, planid))
+
+                tp = None
+                tcip = None
+                page_name = None
+                try:
+                    tc = TestCase(self.env, tcId)
+                    tp = TestPlan(self.env, planid, tcatId)
+                    
+                    page_name = tc['page_name']
+
+                    # Add the test case to the plan, i.e. add a testcaseinplan object
+                    tcip = TestCaseInPlan(self.env, tcId, planid)
+                    if not tcip.exists:
+                        tcip['page_name'] = page_name
+                        if tp['freeze_tc_versions']:
+                            # Set the wiki page version to the current latest version
+                            tcip['page_version'] = tc.wikipage.version
+                        tcip.set_status(self.get_default_tc_status(), author)
+                        tcip.insert()
+                        add_notice(req, _("The Test Case was successfully added to the plan."))
+                    else:
+                        add_warning(req, _("The Test Case was already contained in the specified Test Plan."))
+                    
+                except:
+                    self.env.log.error("Error adding test case to plan!")
+                    self.env.log.error(formatExceptionInfo())
+                    # Back to the test case
+                    add_warning(req, _("An error occurred while adding the test case into the plan."))
+                    req.redirect(req.href.wiki(page_name))
+
+                # Redirect to test case in plan
+                if tp['freeze_tc_versions']:
+                    req.redirect(req.href.wiki(page_name, planid=planid, version=tcip['page_version']))
+                else:
+                    req.redirect(req.href.wiki(page_name, planid=planid))
+                
             elif object_type == 'testcase':
                 req.perm.require('TEST_MODIFY')
                 
@@ -453,10 +512,13 @@ class TestManagerSystem(Component):
                             # Generate a new Id for the next iteration
                             id = self.get_next_id(object_type)
                             pagename = path + '_TC'+str(id)
-                                    
+
+                        add_notice(req, _("The Test Case(s) were successfully pasted into the catalog."))
+                            
                     except:
                         self.env.log.error("Error pasting test cases!")
                         self.env.log.error(formatExceptionInfo())
+                        add_warning(req, _("An error occurred while pasting the test case(s) into the catalog."))
                         req.redirect(req.href.wiki(pagename))
                 
                     # Redirect to test catalog, forcing a page refresh by means of a random request parameter
@@ -482,6 +544,7 @@ class TestManagerSystem(Component):
                     except:
                         self.env.log.error("Error duplicating test case!")
                         self.env.log.error(formatExceptionInfo())
+                        add_warning(req, _("An error occurred while duplicating the test case."))
                         req.redirect(req.href.wiki(tcId))
 
                     # Redirect tp allow for editing the copy test case
@@ -501,7 +564,7 @@ class TestManagerSystem(Component):
                     except:
                         self.env.log.error("Error adding test case!")
                         self.env.log.error(formatExceptionInfo())
-                        add_notice(req, _("Error adding test case"))
+                        add_warning(req, _("An error occurred while adding the test case."))
                         req.redirect(req.path_info)
 
                     # Redirect to edit the test case description
@@ -523,19 +586,53 @@ class TestManagerSystem(Component):
                 self.env.log.debug("About to delete test plan %s on catalog %s" % (planid, catid))
 
                 try:
-                    # Add the new test plan in the database
+                    # Delete the test plan
                     tp = TestPlan(self.env, planid, catid)
                     tp.delete()
 
+                    add_notice(req, _("The Test Plan was deleted successfully."))
+                    
                 except:
                     self.env.log.error("Error deleting test plan!")
                     self.env.log.error(formatExceptionInfo())
                     # Back to the catalog
+                    add_warning(req, _("An error occurred while deleting the test plan."))
                     req.redirect(req.href.wiki(path))
 
                 # Redirect to test catalog, forcing a page refresh by means of a random request parameter
                 req.redirect(req.href.wiki(path, mode=mode, fulldetails=fulldetails, random=str(datetime.now(utc).microsecond)))
+                
+            elif object_type == 'testcaseinplan':
+                req.perm.require('TEST_PLAN_ADMIN')
+                
+                planid = req.args.get('planid')
+                tcId = req.args.get('tcId')
+                id = tcId.rpartition('TC')[2]
+                
+                self.env.log.debug("About to remove test case %s from test plan %s" % (tcId, planid))
 
+                tp = None
+                try:
+                    tp = TestPlan(self.env, planid)
+
+                    # Remove the test case from the plan, i.e. delete the testcaseinplan object
+                    tcip = TestCaseInPlan(self.env, id, planid)
+                    
+                    tcip.delete_history()
+                    tcip.delete()
+
+                    add_notice(req, _("The Test Case was successfully removed from the plan."))
+                    
+                except:
+                    self.env.log.error("Error removing test case from plan!")
+                    self.env.log.error(formatExceptionInfo())
+                    # Back to the test case in plan
+                    add_warning(req, _("An error occurred while removing the test case from the plan."))
+                    req.redirect(req.href.wiki(tcId, planid=planid))
+
+                # Redirect to test plan, forcing a page refresh by means of a random request parameter
+                req.redirect(req.href.wiki(tp['page_name'], planid=planid, random=str(datetime.now(utc).microsecond)))
+                
         elif req.path_info.startswith('/testimport'):
             if req.method == 'POST':
                 if 'import_file' in req.args:
@@ -572,7 +669,7 @@ class TestManagerSystem(Component):
             object_type = req.args.get('type')
             cat_name = req.args.get('cat_name')
             planid = req.args.get('planid', '-1')
-            separator = req.args.get('separator', ',')
+            separator = req.args.get('column_separator', ',')
             fulldetails_str = req.args.get('fulldetails', '')
             raw_wiki_format_str = req.args.get('raw_wiki_format', '')
             
@@ -786,8 +883,6 @@ class TestManagerSystem(Component):
 
     def get_tc_template(self, t_cat_page):
         """ get TestCase template for TestCatalog """
-        db = get_db(self.env)
-        cursor = db.cursor()
 
         try:
             # first get TestCatalog ID
@@ -812,7 +907,7 @@ class TestManagerSystem(Component):
 
     def get_template_by_id(self, t_id):
         """ Returns a template text by its id """
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
 
         try:
@@ -831,7 +926,7 @@ class TestManagerSystem(Component):
 
     def get_template_by_name(self, t_name, t_type):
         """ Get a single template by name and type """
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
         
         try:
@@ -849,10 +944,10 @@ class TestManagerSystem(Component):
 
     # save a template
     def save_template(self, t_id, t_name, t_type, t_desc, t_cont, t_action):
-        db, handle_ta = get_db_for_write(self.env)
-        cursor = db.cursor()
+        @with_transaction(self.env)
+        def do_save_template(db):
+            cursor = db.cursor()
 
-        try:
             if t_action == 'ADD':
                 t_id = self.get_next_template_id()
                 self.env.log.debug("next id is: " + t_id)
@@ -867,41 +962,23 @@ class TestManagerSystem(Component):
                         WHERE id = %s AND name = %s AND type = %s
                 """, (t_desc, t_cont, t_id, t_name, t_type))
 
-            if handle_ta:
-                db.commit()
-                
-        except:
-            self.env.log.error("Error saving template with id '%s'" % t_id)
-            self.env.log.error(formatExceptionInfo())
-            db.rollback()
-            return False
-            
         return True
 
     def remove_template(self, t_id):
         """ Removes a single template by id """
-        db, handle_ta = get_db_for_write(self.env)
-        cursor = db.cursor()
-        
-        try:
+        @with_transaction(self.env)
+        def do_remove_template(db):
+            cursor = db.cursor()
+
             sql = "DELETE FROM testmanager_templates WHERE id = %s"
             self.env.log.debug("removing template with id '%s'" % t_id)
             cursor.execute(sql, (t_id,))
             
-            if handle_ta:
-                db.commit()
-
-        except:
-            self.env.log.error("Error removing template with id '%s'" % t_id)
-            self.env.log.error(formatExceptionInfo())
-            db.rollback()
-            return False
-        
         return True
 
     def get_templates(self, t_type):
         """ Get all templates of desired type """
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
 
         items = []
@@ -921,7 +998,7 @@ class TestManagerSystem(Component):
 
     def template_exists(self, name, t_type):
         """ Check if a given template with desired name and type already exists """
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
         
         try:
@@ -940,7 +1017,7 @@ class TestManagerSystem(Component):
 
     def template_in_use(self, t_id):
         """ Check if a given Test Case template is in use """
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
         
         try:
@@ -961,7 +1038,7 @@ class TestManagerSystem(Component):
 
     def get_next_template_id(self):
         """ Get next id to assign a new temmplate """
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
         ids = []
         try:
@@ -984,7 +1061,7 @@ class TestManagerSystem(Component):
         
         # TODO: Use the TestCatalog class instead
         
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
         cursor.execute("SELECT * from testcatalog")
         items = []
@@ -1381,7 +1458,7 @@ class TestManagerSystem(Component):
         return result
         
     def list_matching_subpages(self, curpage):
-        db = get_db(self.env)
+        db = self.env.get_read_db()
         cursor = db.cursor()
 
         sql = "SELECT w1.name, w1.text, w1.version FROM wiki w1, (SELECT name, max(version) as ver FROM wiki WHERE name LIKE '%s%%' GROUP BY name) w2 WHERE w1.version = w2.ver AND w1.name = w2.name ORDER BY w2.name" % curpage
