@@ -387,7 +387,7 @@ class WikiTestManagerInterface(Component):
     def _get_testplan_properties_markup(self, planid, catid, page_name):
         tp = TestPlan(self.env, planid, catid, page_name)
         
-        result = ''
+        result = u''
         result += _("Author")+': '+html_escape(tp['author'])+'<br />'
         result += _("Created")+': '+format_datetime(tp['time'])+'<br />'
         result += _("Contained Test Cases")+': '+(_("Contains selected Test Cases"), _("Contains all Test Cases"))[tp['contains_all']]+'<br />'
@@ -417,7 +417,7 @@ class WikiTestManagerInterface(Component):
         is_edit = req.args.get('edit_custom', 'false')
         
         has_status = False
-        plan_name = ''
+        plan_name = u''
     
         tc_id = tc_name.partition('_TC')[2]
         test_case = TestCase(self.env, tc_id, tc_name)
@@ -593,7 +593,7 @@ class WikiTestManagerInterface(Component):
         if props is not None:
             obj_props = obj.get_values_as_string(props)
         
-        result = '<input type="hidden" value="' + obj_key + '" id="obj_key_field"></input>'
+        result = u'<input type="hidden" value="' + obj_key + '" id="obj_key_field"></input>'
         result += '<input type="hidden" value="' + obj_props + '" id="obj_props_field"></input>'
         
         result += '<table><tbody>'
@@ -633,7 +633,7 @@ class WikiTestManagerInterface(Component):
         return HTML(result)
 
     def _get_testplan_dialog_markup(self, req, cat_name):
-        result = """
+        result = u"""
             <div id="dialog_testplan" style="padding:20px; display:none;" title="New Test Plan">
                 <form id="new_testplan_form" class="addnew">
                 """ + _("Specify the new Test Plan properties.") + """
@@ -694,7 +694,7 @@ class WikiTestManagerInterface(Component):
         return result
     
     def _get_object_change_history_markup(self, obj, exclude_fields=None):
-        text = '<form id="objectChangeHistory" class="printableform"><fieldset id="objectChangeHistoryFields" class="collapsed"><legend class="foldable" style="cursor: pointer;"><a href="#no6"  onclick="expandCollapseSection(\'objectChangeHistoryFields\')">'+_("Object change history")+'</a></legend>'
+        text = u'<form id="objectChangeHistory" class="printableform"><fieldset id="objectChangeHistoryFields" class="collapsed"><legend class="foldable" style="cursor: pointer;"><a href="#no6"  onclick="expandCollapseSection(\'objectChangeHistoryFields\')">'+_("Object change history")+'</a></legend>'
         
         text += '<table class="listing"><thead>'
         text += '<tr><th>'+_("Timestamp")+'</th><th>'+_("Author")+'</th><th>'+_("Property")+'</th><th>'+_("Previous Value")+'</th><th>'+_("New Value")+'</th></tr>'
@@ -724,7 +724,7 @@ class WikiTestManagerInterface(Component):
         return HTML(text)
     
     def _get_import_dialog_markup(self, req, cat_name):
-        result = """
+        result = u"""
             <div id="dialog_import" style="padding:20px; display:none;" title="Import test cases">
                 <form id="import_file" class="addnew" method="post" enctype="multipart/form-data" action="%s/testimport">
                 """ + _("Select a file in CSV format to import the test cases from.") + """
@@ -783,7 +783,7 @@ class WikiTestManagerInterface(Component):
         return result
     
     def _get_export_dialog_markup(self, req, cat_name, planid, object_type):
-        result = """
+        result = u"""
             <div id="dialog_export" style="padding:20px; display:none;" title="Export test cases">
                 <form id="export_file" class="addnew" method="post" action="%s/testexport">
                 """ + _("Select a name and a location on your machine for the CSV file to export the test cases to.") + """
@@ -847,7 +847,7 @@ class WikiTestManagerInterface(Component):
         return result
     
     def _get_select_testplan_dialog_markup(self, req, tc, tcat):
-        result = """
+        result = u"""
             <div id="dialog_select_testplan" style="padding:20px; display:none;" title='""" + _("Add Test Case to a Test Plan") + """'>
                 <form id="add_to_testplan_form" class="addnew">
                     """ + _("Select the Test Plan to add the Test Case to.") + """
@@ -907,7 +907,7 @@ class WikiTestManagerInterface(Component):
         return result
     
     def _get_error_dialog_markup(self, req):
-        result = """
+        result = u"""
             <div id="dialog_error" style="padding:20px; display:none;" title='""" + _("Error") + """'>
                 %s
             </div>
@@ -920,9 +920,9 @@ class WikiTestManagerInterface(Component):
         add_stylesheet(req, 'testmanager/css/blitzer/jquery-ui-1.8.13.custom.css')
         add_stylesheet(req, 'testmanager/css/testmanager.css')
 
-        before_jquery = 'var baseLocation="'+fix_base_location(req)+'";' + \
+        before_jquery = u'var baseLocation="'+fix_base_location(req)+'";' + \
             'var jQuery_trac_old = $.noConflict(true);'
-        after_jquery = 'var jQuery_testmanager = $.noConflict(true);$ = jQuery_trac_old;jQuery = $;'
+        after_jquery = u'var jQuery_testmanager = $.noConflict(true);$ = jQuery_trac_old;jQuery = $;'
 
         if add_statuses_and_colors and 'TEST_EXECUTE' in req.perm:
             after_jquery += self._get_statuses_and_colors_javascript()
@@ -966,7 +966,7 @@ class WikiTestManagerInterface(Component):
         return common_code
 
     def _get_statuses_and_colors_javascript(self):
-        result = 'var statuses_by_color = {'
+        result = u'var statuses_by_color = {'
 
         testmanagersystem = TestManagerSystem(self.env)
         tc_statuses_by_color = testmanagersystem.get_tc_statuses_by_color()
@@ -1010,7 +1010,7 @@ class WikiTestManagerInterface(Component):
             if tc == cat_name:
                 break
 
-        text = ''
+        text = u''
 
         text +='<div>'
         text += self._render_breadcrumb(breadcrumb, planid, mode, fulldetails)
@@ -1036,7 +1036,7 @@ class WikiTestManagerInterface(Component):
 
         # Generate the markup
         ind = {'count': 0, 'totals': None}
-        text = ''
+        text = u''
 
         if mode == 'tree':
             text +='<div style="padding: 0px 0px 10px 10px">'+_("Filter:")+' <input id="tcFilter" title="'+_("Type the test to search for, even more than one word. You can also filter on the test case status (untested, successful, failed).")+'" type="text" size="40" onkeyup="starthighlight(\'ticketContainer\', this.value)"/>&nbsp;&nbsp;<span id="ticketContainer_searchResultsNumberId" style="font-weight: bold;"></span></div>'
@@ -1089,7 +1089,7 @@ class WikiTestManagerInterface(Component):
 
         # Generate the markup
         ind = {'count': 0, 'totals': None}
-        text = ''
+        text = u''
         
         if mode == 'tree':
             text +='<div style="padding: 0px 0px 10px 10px">'+_("Filter:")+' <input id="tcFilter" title="'+_("Type the test to search for, even more than one word. You can also filter on the test case status (untested, successful, failed).")+'" type="text" size="40" onkeyup="starthighlight(\'ticketContainer\', this.value)"/>&nbsp;&nbsp;<span id="ticketContainer_searchResultsNumberId" style="font-weight: bold;"></span></div>'
@@ -1140,7 +1140,7 @@ class WikiTestManagerInterface(Component):
         markup, num_plans = self._render_testplan_list(catid, mode, fulldetails, show_delete_button)
 
 
-        text = '<form id="testPlanList" class="printableform"><fieldset id="testPlanListFields" class="collapsed"><legend class="foldable" style="cursor: pointer;"><a href="#no4"  onclick="expandCollapseSection(\'testPlanListFields\')">'+_("Available Test Plans")+' ('+str(num_plans)+')</a></legend>'
+        text = u'<form id="testPlanList" class="printableform"><fieldset id="testPlanListFields" class="collapsed"><legend class="foldable" style="cursor: pointer;"><a href="#no4"  onclick="expandCollapseSection(\'testPlanListFields\')">'+_("Available Test Plans")+' ('+str(num_plans)+')</a></legend>'
         text +='<div style="padding: 0px 0px 10px 10px">'+_("Filter:")+' <input id="tpFilter" title="'+_("Type the test to search for, even more than one word.")+'" type="text" size="40" onkeyup="starthighlightTable(\'testPlanListTable\', this.value)"/>&nbsp;&nbsp;<span id="testPlanListTable_searchResultsNumberId" style="font-weight: bold;"></span></div>'
         text += markup
         text += '</fieldset></form>'
@@ -1154,7 +1154,7 @@ class WikiTestManagerInterface(Component):
 
         cat = TestCatalog(self.env, catid)
         
-        result = '<table class="listing" id="testPlanListTable"><thead>'
+        result = u'<table class="listing" id="testPlanListTable"><thead>'
         result += '<tr><th>'+_("Plan Name")+'</th><th>'+_("Author")+'</th><th>'+_("Timestamp")+'</th><th>'+_("Contained Test Cases")+'</th><th>'+_("Test Case Versions")+'</th><th></th></tr>'
         result += '</thead><tbody>'
         
@@ -1188,7 +1188,7 @@ class WikiTestManagerInterface(Component):
         else:
             display_breadcrumb = 'block'
         
-        text = '<span style="display: %s">' % display_breadcrumb
+        text = u'<span style="display: %s">' % display_breadcrumb
         path_len = len(breadcrumb)
         for i, x in enumerate(breadcrumb):
             if i == 0:
@@ -1211,7 +1211,7 @@ class WikiTestManagerInterface(Component):
     # Render the subtree
     def _render_subtree(self, planid, component, ind, level):
         data = component
-        text = ''
+        text = u''
         if (level == 0):
             data = component['childrenC']
             text +='<ul style="list-style: none;">';
@@ -1260,7 +1260,7 @@ class WikiTestManagerInterface(Component):
         
         tc_target = ("", " target='_blank'")[self.open_new_window]
         
-        text=''
+        text=u''
         sortedList = sorted(data)
         for x in sortedList:
             tick = data[x]
@@ -1313,7 +1313,7 @@ class WikiTestManagerInterface(Component):
         else:
             statusLabel = _("Unknown")
         
-        text = ''
+        text = u''
         text += '<img style="display: '+display['green']+';" id="tcTitleStatusIcongreen" src="../chrome/testmanager/images/green.png" title="'+_(statusLabel)+'"></img></span>'
         text += '<img style="display: '+display['yellow']+';" id="tcTitleStatusIconyellow" src="../chrome/testmanager/images/yellow.png" title="'+_(statusLabel)+'"></img></span>'
         text += '<img style="display: '+display['red']+';" id="tcTitleStatusIconred" src="../chrome/testmanager/images/red.png" title="'+_(statusLabel)+'"></img></span>'
@@ -1323,7 +1323,7 @@ class WikiTestManagerInterface(Component):
     # Render the subtree as a tree table
     def _render_subtree_as_table(self, context, planid, component, ind, level, table_columns=None, table_columns_map=None, custom_ctx=None, fulldetails=False):
         data = component
-        text = ''
+        text = u''
 
         if (level == 0):
             data = component['childrenC']
@@ -1379,7 +1379,7 @@ class WikiTestManagerInterface(Component):
 
         tc_target = ("", " target='_blank'")[self.open_new_window]
         
-        text=''
+        text=u''
         sortedList = sorted(data)
         for x in sortedList:
             tick = data[x]
@@ -1473,7 +1473,7 @@ class WikiTestManagerInterface(Component):
         return text
 
     def _render_totals(self, table_columns, totals):
-        text = ''
+        text = u''
     
         for col in table_columns:
             if col['visible'] == 'True':
@@ -1523,7 +1523,7 @@ class WikiTestManagerInterface(Component):
             if len(tc_statuses_by_color[color]) > 1:
                 need_menu = True
 
-        text = ''
+        text = u''
 
         if need_menu:
             text += '<div id="copyright" style="display: none;">Copyright &copy; 2010 <a href="http://apycom.com/">Apycom jQuery Menus</a></div>'
@@ -1635,7 +1635,7 @@ class WikiTestManagerInterface(Component):
         self.env.log.debug("<<< _update_totals: %s", totals)
                         
     def _get_custom_fields_columns(self, obj, table_columns, table_columns_map, fields):
-        result = ''
+        result = u''
         
         for f in fields:
             if table_columns_map[f['name']]['visible'] == 'True':
